<?php

class RestOpen {

    /**
     * @var string $username 
     */
    protected $username;

    /**
     * @var string $password 
     */
    protected $password;

    /**
     * @var string $token 
     */
    protected $token;

    /**
     * @var integer $token 
     */
    protected $workspaceId;

    /**
     * @var string $apiUrl 
     */
    protected $apiUrl = 'https://open.relatify.com/';

    /**
     * @brief set some settings for the api
     */
    public function __construct($username, $password, $workspaceId) 
    {
        $this->setUsername($username);
        $this->setPassword($password);
        $this->setWorkspaceId($workspaceId);
    }

    /**
     * @param mixed $username
     */
    public function setUsername($username) 
    {
        $this->username = $username;
    }

    /**
     * @return mixed
     */
    public function getUsername() 
    {
        return $this->username;
    }

    /**
     * @param mixed $password
     */
    public function setPassword($password) 
    {
        $this->password = $password;
    }

    /**
     * @return mixed
     */
    public function getPassword() 
    {
        return $this->password;
    }

    /**
     * @param string $url
     */
    public function setApiUrl($url) 
    {
        $l = strlen($url);

        if($l >0 && $url[$l-1] != '/') 
        {
            $url .= '/';
        }

        $this->apiUrl = $path;
    }

    /**
     * @return mixed 
     */
    public function getApiUrl() 
    {
        if(is_null($this->apiUrl) || $this->apiUrl == '') {
            return false;
        }

        return $this->apiUrl;
    }

    /**
     * @param integer $workspaceId
     */
    public function setWorkspaceId($workspaceId) 
    {
        $this->workspaceId = (int) $workspaceId;
    }

    /**
     * @return integer 
     */
    public function getWorkspaceId() 
    {
        return $this->workspaceId;
    }

    /**
     * @param mixed 
     */
    protected function setToken($token) 
    {
        $this->token = $token;
    }

    /**
     * @return string
     */
    public function getToken() 
    {
        return $this->token;
    }

    /*
     * @brief set token or/and return the data of the authorization
     */
    public function auth() 
    {
        $data = $this->connect('api/auth', array('workspaceId' => $this->getWorkspaceId()), 'post');

        if($data->valid) 
        {
            $this->setToken($data->data->token);
        }

        return $data;
    } 


    /**
     * @brief method for connecting to the open rest api
     * @param string    $call
     * @param array     $urlParams
     * @param string    $type
     * @param array     @saveData
     * @return          stdClass
     */
    public function connect($call, $urlParams = array(), $type = 'get', $saveData = array()) 
    {
        $type = strtoupper($type);
        $apiUrl = $this->getApiUrl();

        //check on correct request type
        if(!in_array($type, array('GET', 'POST', 'DELETE', 'PUT'))) {
            return $this->criticalFailure('Wrong request type', 'Please use one of the following request types: GET, POST, DELETE, PUT');
        }

        //check if we have a apiUrl
        if(!$apiUrl) 
        {
            return $this->criticalFailure('Open application url not valid.', 'Open application url is not valid. (' . $apiUrl . ')');    
        }

        //create the url we're going to call
        //$urlParams = array_map('urlencode', $urlParams);
        $uri = call_user_func_array('sprintf', array_merge((array) $call, $urlParams ));
        $url = $apiUrl . $uri;
        $url = str_replace(' ', '%20', $url);

        //check if we already have a token, if not lets login
        $token = $this->getToken();
        if(is_null($token) && !strstr($call, 'api/auth')) 
        {
            $auth = $this->auth();
            if(!$auth->valid) 
            {
                return $this->criticalFailure($auth->data->message, $auth->data->extra);
            }
        }

        //if we have a auth call we need to add a workspaceId if its a get we need to add the token to the url
        if(strstr($call, 'api/auth')) 
        {
            $saveData['workspaceId'] = $this->getWorkspaceId();
        } elseif($type == 'GET') {
            $extraQuery = array('token'=> $this->getToken());           
            $url .= '?' . http_build_query( $extraQuery );
        }

        //set some curl options
        $c = curl_init();
        $curlOptions = array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_SSL_VERIFYPEER => false
        );

        //if its not a get request we need to add the token and set some extra options
        if($type != 'GET') 
        {
            $headers = array(
                'X-Auth-Token: ' . $this->getToken(),
                'Content-Type: application/json'
            );

            $curlOptions[CURLOPT_HTTPHEADER] = $headers;
            $curlOptions[CURLOPT_CUSTOMREQUEST] = $type;
            $curlOptions[CURLOPT_POSTFIELDS] = json_encode($saveData);
        }

        //if the call is a authorization then we need to add the login data
        if(strstr($call, 'api/auth')) 
        {
            $login = array(
                $this->getUsername(),
                $this->getPassword()
            );
            
            $curlOptions[CURLOPT_USERPWD] = implode(':', $login);
            $curlOptions[CURLOPT_HTTPAUTH] = CURLAUTH_BASIC;
        }

        //set option and execute the call
        curl_setopt_array($c, $curlOptions);
        $result = curl_exec($c);

        //decode result and get some info from the call
        $json = json_decode($result);
        $httpCode = curl_getinfo($c, CURLINFO_HTTP_CODE);

        curl_close($c);
        
        //to always get json back we first check if we have a valid json. If not then create one.
        if(!is_object($json)) 
        {
            $json = $this->criticalFailure();
        } else if($httpCode != 200 && $httpCode != 201) 
        {
            $json = $this->criticalFailure(isset($json->data->message) ? $json->data->message : null, isset($json->data->extra) ? $json->data->extra : '');
        }

        return $json;
    }


    /**
     * @brief we always want a stdClass back. So lets create a stdClass when there is a problem.
     * @param string $msg
     * @param string $extra
     * @return stdClass
     */
    protected function criticalFailure($msg = null, $extra = null) 
    {
        $json = new stdClass();
        $json->valid = false;
        $json->data = new stdClass();
        $json->data->message = is_null($msg) ? "Critical failure" : $msg;
        if($extra !== '') {
            $json->data->extra = is_null($extra) ? "Result was not in valid JSON format" : $extra;
        }
        
        return $json;
    }

    /**
     * @brief get the subscribers out of a selection (per page)
     * @param integer $selectionId
     * @param integer $page
     * @param integer $rows
     * @return stdClass
     */
    public function getSelectionAllRows( $selectionId, $page = 0, $rows = 500 ) 
    {
        return $this->connect(
            'api/selection/%u/page/%u/rows/%u',
            array(
                (int) $selectionId,
                (int) $page,
                (int) $rows
            )
        );
    }

    /**
     * @brief get a random row of a data table 
     * @param integer $selectionId
     * @return stdClass
     */
    public function getSelectionRandomRow($selectionId) 
    {
        return $this->connect(
            'api/selection/%u/random', 
            array(
                (int) $selectionId
            )
        );
    }

    /**
     * @brief get the first row of a data table based on a email address
     * @param integer $selectionId
     * @param string $email
     * @return stdClass
     */
    public function getSelectionFirstRowByEmail($selectionId, $email) 
    {
        return $this->connect(
            'api/selection/%u/email-address/%s', 
            array(
                (int) $selectionId,
                $email
            )
        );
    }

    /**
     * @brief get the list of data tables in this workspace
     * @return stdClass
     */
    public function getDataTableList() 
    {
        return $this->connect('api/datatable/list');
    }

    /**
     * @brief get the the details of a data table 
     * @param string $tableName
     * @return stdClass
     */
    public function getDataTableDetails($tableName = '') 
    {
        return $this->connect('api/datatable/details/%s', array(
            $tableName
        ));
    }

    /**
     * @brief search the data table for a specific value and return specific columns
     * @param string $tableName
     * @param string $fieldName
     * @param string $fieldValue
     * @param integer $page
     * @param array $columns
     * @return stdClass
     */
    public function getDataTableSearch($tableName, $fieldName, $fieldValue, $page = null, $columns = array()) 
    {
        $path = 'api/datatable/search/%s/%s/%s';
        $options = array(
            $tableName,
            $fieldName,
            $fieldValue
        );

        if(!is_null($page)) 
        {
            $path .= '/%s';
            $options = array_merge($options, array((int) $page));
        }

        if(count($columns)) {
            $path .= '/columns/%s';
            $options = array_merge($options, array(json_encode($columns)));
        }

        return $this->connect($path, $options);
    }


    /**
     * @brief Update a row in a spefic data table based on the primary key 
     * @param string $tableName
     * @param string $primaryKey
     * @param array $search
     * @return stdClass
     */
    public function putDataTableUpdateRow($tableName, $primaryKey, $search) 
    {
        return $this->connect('api/datatable/update/%s/%s', array(
                $tableName,
                $primaryKey,
            ), 
            'PUT', 
            (array) $search
        );
    }

    /**
     * @brief create a new row in a data table 
     * @param string $tableName
     * @param array $search
     * @return stdClass
     */
    public function postDataTableNewRow($tableName, $search) {
        return $this->connect('api/datatable/create/%s', array($tableName),
            'POST',
            (array) $search
        );
    }

    /**
     * @brief delete a row in a data table 
     * @param string $tableName
     * @param string $primaryKey 
     * @return stdClass
     */
    public function deleteDataTableRow($tableName, $primaryKey) {
        return $this->connect('api/datatable/remove/%s/%s', array(
            $tableName,
            $primaryKey
        ), 'DELETE');
    }
}
